;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: bosco -*-
;;;; ***********************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          main.lisp
;;;; Project:       Bosco - Application framework for OSX and OpenMCL
;;;; Purpose:       Bosco 0.6 main program
;;;; Programmer:    mikel evins
;;;; Date Started:  05/12/2004
;;;;
;;;; $Id: main.lisp,v 1.2 2006/12/14 00:12:36 mikel Exp $
;;;; ***********************************************************************

(in-package "BOSCO")

;;; ------------------------------------------------------------
;;; redefinitions of the document code from lib/application.lisp
;;; ------------------------------------------------------------

;;; ------------------------------------------------------------
;;; the Objective-C document class

(defclass my-document (ns:ns-document)
  ((mstring :foreign-type :id)
   (mtextview :foreign-type :id))
  (:metaclass ns:+ns-object))

;;; ------------------------------------------------------------
;;; document methods

;;; initialization

(define-objc-method ((:id init) my-document)
  (let ((doc (send-super 'init)))
    (setf (slot-value self 'mstring)
          (send (send (@class ns-attributed-string) 'alloc) :init-with-string #@""))
    doc))

;;; identify the document's nib

(define-objc-method ((:id window-nib-name) my-document)
  #@"MyDocument")


;;; initialize the document after loading the nib

(define-objc-method ((:void :window-controller-did-load-nib controller) my-document)
    (send-super :window-controller-did-load-nib controller)
  (send (send (slot-value self 'mtextview) 'text-storage)
        :set-attributed-string (send self 'string)))

;;; getters and setters for the string data

(define-objc-method ((:id string) my-document)
  (slot-value self 'mstring))

(define-objc-method ((:void :set-string newvalue) my-document)
    (unless (eql newvalue (slot-value self 'mstring))
      (unless (%null-ptr-p (slot-value self 'mstring))
        (send (slot-value self 'mstring) 'release))
      (setf (slot-value self 'mstring) 
            (send newvalue 'copy))))

;;; text delegate methods

(define-objc-method ((:void text-did-change) my-document)
  (send self :set-string (send (slot-value self 'mtextview) 'text-storage)))

;;; document archiving and unarchiving

(define-objc-method ((:id :data-representation-of-type type)
                     my-document)
	(declare (ignorable type))
  (send self :set-string (send (slot-value self 'mtextview) 'text-storage))
  (send (@class ns-archiver) :archived-data-with-root-object (send self 'string)))

(define-objc-method ((:<BOOL> :load-data-representation data
							  :of-type type)
					 my-document)
  (declare (ignorable type))
  (send self :set-string 
        (send (@class ns-unarchiver) :unarchive-object-with-data data))
  #$YES)

;;; the main function of your application
;;; the toplevel-function runs the Cocoa NSApp object

(defmethod toplevel-function ((app bosco-cocoa-application) init-file)
  (declare (ignore init-file))
  (with-autorelease-pool
    (or *NSApp* (setq *NSApp* (init-cocoa-application)))
    (run-event-loop)))
